package domain;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

public abstract class MatrixN {
	private int[][] matrix;
	
	public MatrixN(int dim) throws MatrixException {
		if (dim <= 0)
			throw new MatrixException("Fehler: MatrixN muss mind. 1 DIM haben!");
		
		matrix = new int[dim][dim];
	}
	
	public int getDimension() {
		return matrix.length;
	}
	
	public int get(int row, int col) throws MatrixException {
		try {
			return matrix[row][col];
		}
		catch (IndexOutOfBoundsException e) {
			throw new MatrixException("Fehler: ungültige Position!");
		}
	}

	public void set(int row, int col, int value) throws MatrixException {
		try {
			matrix[row][col] = value;
			matrix[col][row] = value;
		}
		catch (IndexOutOfBoundsException e) {
			throw new MatrixException("Fehler: ungültige Position!");
		}
	}
	
	@Override
	public boolean equals(Object other) {
		if (other == null)
			return false;
		if (this == other)
			return true;
		if ( ! (other instanceof MatrixN) )
			return false;
		MatrixN o = (MatrixN)other;
		if (o.matrix.length != matrix.length)
			return false;
		
		for (int r=0; r<matrix.length; r++)
			for (int s=0; s<matrix[r].length; s++)
				if (matrix[r][s] != o.matrix[r][s])
					return false;
		return true;
	}
	
	@Override
	public String toString() {
		String erg = "";
		for (int[] zeile : matrix) {
			for (int element : zeile) {
				erg += element + " ";
			}
			erg += "\n";
		}
		/*for (int r=0; r<matrix.length; r++) {
			for (int c=0; c<matrix[r].length; c++) {
				erg += matrix[r][c] + " ";
			}
			erg += "\n";
		}*/
		return erg;
	}
	
	public void clone(MatrixN destination) throws MatrixException {
		if (destination == null)
			throw new MatrixException("Fehler: keine Matrix!");
		if (matrix.length != destination.matrix.length)
			throw new MatrixException("Fehler: unterschiedliche Größe!");
		
		for (int r=0; r<matrix.length; r++) {
			for (int c=0; c<matrix[r].length; c++) {
				destination.set(r, c, matrix[r][c]);
			}
		}
	}
	

	public void importMatrixN(String filename) throws MatrixException {
		if (filename == null)
			throw new MatrixException("Fehler: kein Dateiname!");
		try {
			String zeile;
			String []zei;
			int z;
			BufferedReader br = new BufferedReader(new FileReader(filename));
			zeile = br.readLine();
			z = 0;
			zei = zeile.trim().split(";");
			matrix = new int[zei.length][zei.length];
			while (zeile != null) {
				zei = zeile.trim().split(";");
				for (int s=0; s<zei.length; s++) {
					set(z, s, Integer.parseInt(zei[s].trim()));
				}
				zeile = br.readLine();
				z++;
			}
			br.close();
		}
		catch (NumberFormatException e) {
			throw new MatrixException("Fehler: keine Zahl!");			
		}
		catch (FileNotFoundException e) {
			throw new MatrixException("Fehler: ungültiger Pfad!");			
		}
		catch (IOException e) {
			throw new MatrixException("Fehler: beim Lesen der Datei!");
		}
	}
}
















